#ifndef _XsmKnobP_h
#define _XsmKnobP_h

/* include the public header as well as the super class private header */

#include "Knob.h"
#include <Xm/PrimitiveP.h>

/*--------------------------------------------------------------------------*/
/*************************
 * THE KNOB CLASS RECORD *
 *************************/

/* Define the types of procedure pointers not defined in the Intrinsics. */

typedef void (*XsmGetDiametersProc)(Widget, unsigned int *, unsigned int *) ;

/* Define inheritance macros for all knob class procedures. */

#define XsmInheritGetDiameters ((XsmGetDiametersProc) _XtInherit)
#define XsmInheritCreateSegments ((XtWidgetProc) _XtInherit)
#define XsmInheritDrawIndicator ((XtWidgetProc) _XtInherit)
#define XsmInheritTurn ((XtTimerCallbackProc) _XtInherit)
#define XsmInheritDraw ((XtWidgetProc) _XtInherit)

/*
 * Define class part record.  Be sure to include an extension pointer
 * field for future expansion.
 */

typedef struct {
    XsmGetDiametersProc  get_diameters;   /* procedure to inner and
                                             outer circles */
    XtWidgetProc         create_segments; /* procedure used to create
                                             handle segments */
    XtWidgetProc         draw_indicator;  /* procedure for drawing knob
                                             indicator */
    XtTimerCallbackProc  turn;            /* timeout procedure used to
                                             turn knob */
    XtWidgetProc         draw;            /* procedure for drawing the knob */
    XtPointer            extension;       /* pointer to extension record
                                             used for future expansion */
} XsmKnobClassPart;

/*
 * Define class record which include all super class parts.
 * Start first with Core, then Primitive, and finally the Knob
 * class part defined above.
 */

typedef struct _XsmKnobClassRec {
    CoreClassPart core_class;
    XmPrimitiveClassPart primitive_class;
    XsmKnobClassPart knob_class;
} XsmKnobClassRec;

/*--------------------------------------------------------------------------*/
/****************************
 * THE KNOB INSTANCE RECORD *
 ****************************/

/*
 * Define the instance part record.  Be sure to include all of the
 * fields to hold resource values as well as fields for internal variables
 * unique to each instance of a knob widget.
 */

typedef struct _XsmKnobPart {
    XtCallbackList value_changed_callback; /* resource - callback when value
                                              changes */
    int value;                  /* resource - value of the knob */
    int max_val;                /* resource - maximum value allowed */
    int min_val;                /* resource - minimum value allowed */
    int turn_delay;             /* resource - the delay before the next
                                              increment */
    Pixel knob_color;           /* resource - the color of the knob */
    Pixel indicator_color;      /* resource - the color of the indicator */
    Dimension margin_width;     /* resource - determines the outer edge of */
    Dimension margin_height;    /*            the circle. Whichever value is
                                              greater when the dimension is
                                              subtracrted from its margin
                                              value determines the knob
                                              diameter. */
    Dimension knob_margin;      /* resource - The percentage from the knob's
                                              outer edge to draw the inner
                                              inner circle. */
    Dimension orig_width;       /* saves the original dimension */
    Dimension orig_height;      /* saves the original dimension */
    Boolean move_clockwise;     /* indicates direction of turn */
    Boolean turning;            /* indicates whether turning is in progress */
    GC gc;                      /* graphics context use in graphic ops */
    XSegment *segments;         /* the line segments used for drawing the
                                   handle */
    int num_segments;           /* the number of segments in the above list */
    XtIntervalId timer_id;      /* the timer id of the timeout used in turning
                                   the knob */
    double angle_offset;        /* used to calculate the segment positions */
} XsmKnobPart;

/*
 * Define instance record which include all super class instance parts.
 * Start first with Core, then Primitive, and finally the Knob
 * instance part defined above.
 */

typedef struct _XsmKnobRec {
    CorePart        core;
    XmPrimitivePart primitive;
    XsmKnobPart     knob;
} XsmKnobRec;

#endif /* _XsmKnobP_h */
/* DON't ADD STUFF AFTER THIS #endif */
